/*
 * Filename : cstc_lcd.h
 * Hardware : Controller	-> P89V51RD2
 *            XTAL			-> 11.0592 MHz
   I/O			  
			  PORT			->P2
			  EN			->P2^4
			  RS			->P2^5
			  D4-D7			->P2^0-P2^3
 * Compiler : Keil Vision3            
 * Author   : Mr.Noppol Insorn
 			  E-Mail		->inoppol@gmail.com
 * Date		: 2011/06/14
 *Use with LCD 16X2 Display
 *lcd_out(lineNumber,string); Ex. LCD_out(0,"Hello");
 */

 sbit LCD_en=P2^4;
 sbit LCD_rs=P2^5;
 #define LCD_DELAY 230 /* Delay for 1 ms */
 #define LCD_clear() LCD_command(0x1)	/* Clear display LCD */
 #define LCD_origin() LCD_command(0x2)	/* Set to origin LCD */
 #define LCD_row1() LCD_command(0x80)	/* Begin at Line 1 */
 #define LCD_row2() LCD_command(0xC0)   /* Begin at Line 2 */
 
/***************************************************
 * Prototype(s)                                    *
 ***************************************************/
void LCD_delay(unsigned char ms);
void LCD_enable();
void LCD_command(unsigned char command);
void LCD_putc(unsigned char ascii);
void LCD_puts(unsigned char *lcd_string);
void LCD_init();
void LCD_out(char line,unsigned char *lcd_string);
void LCD_put_cg(unsigned address,unsigned char *lcd_cg);
void LCD_goto(unsigned char x,unsigned char y);

/***************************************************
 * Sources                                         *
 ***************************************************/
void LCD_out(char line,unsigned char *lcd_string){
	if(line==0)
	LCD_row1();
	else if(line==1)
	LCD_row2();

	LCD_puts(lcd_string);
}

void LCD_delay(unsigned char ms)
{
	unsigned char n;
	unsigned int i;
	for (n=0; n<ms; n++)
	{
		for (i=0; i<LCD_DELAY; i++); /* For 1 ms */
	}
    
}

void LCD_enable()
{
    LCD_en = 0; /* Clear bit P2.4 */
    LCD_delay(1);
    LCD_en = 1; /* Set bit P2.4 */
}

void LCD_command(unsigned char command)
{
    LCD_rs = 0; /* Clear bit P2.5 */
    P2 = (P2 & 0xF0)|((command>>4) & 0x0F);
    LCD_enable();
    P2 = (P2 & 0xF0)|(command & 0x0F);
    LCD_enable();
    LCD_delay(1);
}

void LCD_putc(unsigned char ascii)
{
    LCD_rs = 1; /* Set bit P2.5 */
    P2 = (P2 & 0xF0)|((ascii>>4) & 0x0F);
    LCD_enable();
    P2 = (P2 & 0xF0)|(ascii & 0x0F);
    LCD_enable();
    LCD_delay(1);
}

void LCD_puts(unsigned char *lcd_string)
{
	while (*lcd_string) 
	{
		LCD_putc(*lcd_string++);
	}
}

void LCD_goto(unsigned char x,unsigned char y){
	if(x==0){
		LCD_command(0x80|y);
	}
	else if(x==1){
		LCD_command(0xC0|y);
	}
}

void LCD_init()
{
    LCD_en = 1; /* Set bit P2.4 */
    LCD_rs = 0; /* Clear bit P2.5 */   
    LCD_command(0x33);
    LCD_command(0x32);
    LCD_command(0x28);
    LCD_command(0x0C);
    LCD_command(0x06);
    LCD_command(0x01); /* Clear */
    LCD_delay(256);
}
